<?php
/**
 * Translate
 *
 * @package   WP Grid Builder - Multilingual
 * @author    Loïc Blascos
 * @copyright 2019-2020 Loïc Blascos
 */

namespace WP_Grid_Builder_Multilingual\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handle strings translation
 *
 * @class WP_Grid_Builder_Multilingual\Includes\Translate
 * @since 1.0.0
 */
class Translate {

	use Helpers;

	/**
	 * Constructor
	 *
	 * @access public
	 * @since 1.0.0
	 */
	public function __construct() {

		add_filter( 'wp_grid_builder/grid/settings', [ $this, 'translate_settings' ], PHP_INT_MAX );
		add_filter( 'wp_grid_builder/card/settings', [ $this, 'translate_settings' ], PHP_INT_MAX );
		add_filter( 'wp_grid_builder/facet/settings', [ $this, 'translate_settings' ], PHP_INT_MAX );
		add_filter( 'wp_grid_builder/grid/query_args', [ $this, 'prepare_tax_query' ], PHP_INT_MAX );

	}

	/**
	 * Get registry from context (grid/card/facet)
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param string $context Context of the registry.
	 * @return array
	 */
	public function get_registry( $context ) {

		return apply_filters( 'wp_grid_builder_i18n/' . $context . '/register_strings', [] );

	}

	/**
	 * Translate strings recursively
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array  $strings  Holds strings to translate.
	 * @param string $context  String context.
	 * @param array  $registry Holds registered string to translate.
	 * @param string $lang     Language code.
	 * @return array
	 */
	public function translate_strings( $strings, $context, $registry, $lang ) {

		foreach ( $strings as $key => $string ) {

			if ( is_array( $string ) ) {
				$strings[ $key ] = $this->translate_strings( $string, $context, $registry, $lang );
			} elseif ( isset( $registry[ $key ] ) ) {

				// Set a default domain if missing.
				$domain = ! empty( $registry[ $key ]['domain'] ) ? $registry[ $key ]['domain'] : 'Gridbuilder ᵂᴾ';
				// Apply filters to translate from Polylang or WPML.
				$new_string = apply_filters( 'wp_grid_builder_i18n/translate_string', $string, $key, $context, $domain, $lang );

				if ( $new_string !== $string ) {
					$strings[ $key ] = $new_string;
				}
			}
		}

		return $strings;

	}

	/**
	 * Translate grid/card/facet settings
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array $settings Holds grid settings.
	 * @return array
	 */
	public function translate_settings( $settings ) {

		$context  = explode( '/', current_filter() )[1];
		$registry = $this->get_registry( $context );
		$language = $this->get_current_language();

		return $this->translate_strings( $settings, $context, $registry, $language );

	}

	/**
	 * Prepare tax_query (convert term taxonomy ids)
	 * WPML and Polylang do not handle translation if taxonomy parameter is missing in tax_query...
	 * However, taxonomy parameter is optional in WP_Tax_QUery when field=term_taxonomy_id
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array $query_args Holds query arguments.
	 * @return array
	 */
	public function prepare_tax_query( $query_args ) {

		$current = $this->get_current_language();
		$default = $this->get_default_language();

		// No auto translate will occur in this case.
		if ( $current === $default ) {
			return $query_args;
		}

		// We let Polylang or WPML translate query.
		unset( $query_args['lang'] );

		if ( empty( $query_args['tax_query'] ) ) {
			return $query_args;
		}

		$query_args['tax_query'] = array_map(
			[ $this, 'convert_tax_query_clauses' ],
			(array) $query_args['tax_query']
		);

		return $query_args;

	}

	/**
	 * Recursively process each clause of tax_query
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array $clauses Holds tax_query clauses.
	 * @return array
	 */
	public function convert_tax_query_clauses( $clauses ) {

		if ( ! is_array( $clauses ) || ! empty( $clauses['taxonomy'] ) ) {
			return $clauses;
		}

		if ( ! isset( $clauses['terms'] ) ) {
			return array_map( __METHOD__, $clauses );
		}

		if ( ! isset( $clauses['field'] ) || 'term_taxonomy_id' !== $clauses['field'] ) {
			return $clauses;
		}

		return $this->convert_tax_query_clause( $clauses );

	}

	/**
	 * Convert tax_query clause to add missing taxonomy parameter
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array $clause Holds tax_query clause.
	 * @return array
	 */
	public function convert_tax_query_clause( $clause ) {

		$clauses = [];

		foreach ( $this->get_term_terms( $clause['terms'] ) as $term ) {

			$clauses[] = wp_parse_args(
				[
					'field'    => 'term_id',
					'terms'    => (array) $term->term_id,
					'taxonomy' => $term->taxonomy,
				],
				$clause
			);
		}

		if ( ! empty( $clause['relation'] ) ) {
			$clauses['relation'] = $clause['relation'];
		}

		return $clauses;

	}

	/**
	 * Query terms from term taxonomy ids.
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array $term_taxonomy_ids Holds term taxonomy ids.
	 * @return array
	 */
	public function get_term_terms( $term_taxonomy_ids ) {

		$terms = get_terms(
			[
				'term_taxonomy_id' => $term_taxonomy_ids,
				'lang' => $this->get_default_language(),
			]
		);

		if ( is_wp_error( $terms ) || empty( $terms ) ) {
			return [];
		}

		return $terms;

	}
}
